unit formPTChart;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Data.DB,
  VCLTee.TeEngine, VCLTee.TeeURL, VCLTee.TeeSeriesTextEd, VCLTee.Series,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, VCLTee.DBChart, VCLTee.TeeTools,
  Vcl.StdCtrls, Vcl.buttons, VCLTee.TeeMagnifyTool, VCLTee.TeeScrollPagerTool, System.StrUtils,
  system.Math;

type
  TfrmPTChart = class(TForm)
    DBChart1: TDBChart;
    Series1: TLineSeries;
    Series2: TLineSeries;
    SeriesTextSource1: TSeriesTextSource;
    SeriesTextSource2: TSeriesTextSource;
    Series3: TLineSeries;
    SeriesTextSource3: TSeriesTextSource;
    SeriesTextSource4: TSeriesTextSource;
    cbxSyncDep: TCheckBox;
    ChartTool1: TAxisScrollTool;
    ChartTool2: TAxisScrollTool;
    ChartTool3: TAxisScrollTool;
    ChartTool4: TAxisScrollTool;
    Panel1: TPanel;
    cbxLogTop: TCheckBox;
    cbxLogBottom: TCheckBox;
    cbxAutoTop: TCheckBox;
    cbxAutoBottom: TCheckBox;
    cbxInvertTop: TCheckBox;
    cbxInvertBottom: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure DBChart1BeforeDrawAxes(Sender: TObject);
    procedure DBChart1GetAxisLabel(Sender: TChartAxis;
      Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
    procedure cbxSyncDepClick(Sender: TObject);
    procedure cbxLogTopClick(Sender: TObject);
    procedure cbxLogBottomClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBChart1Scroll(Sender: TObject);
    procedure DBChart1UndoZoom(Sender: TObject);
    procedure cbxInvertTopClick(Sender: TObject);
    procedure cbxInvertBottomClick(Sender: TObject);
    procedure cbxAutoBottomClick(Sender: TObject);
    procedure cbxAutoTopClick(Sender: TObject);

  private
    procedure SyncTopAxis;
    procedure ApplyChartLogLabels (bAuto: boolean);
    procedure CalculateLabels(AAxis: TChartAxis; AInc: integer; bAuto: boolean);
    procedure GenerateLabelArray(AInc: integer);

    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPTChart: TfrmPTChart;
  AVal: array of double;
  bFirstTime: boolean;

implementation

{$R *.dfm}

// ========== TfrmPTChart.FormCreate ===========================================
procedure TfrmPTChart.FormCreate(Sender: TObject);
begin
  FormatSettings.DecimalSeparator:='.';
  FormatSettings.ThousandSeparator:=',';
end;

// ========== TfrmPTChart.FormShow =============================================
procedure TfrmPTChart.FormShow(Sender: TObject);
var
  i: integer;
  LStrList: TStringList;

begin
// Load Pressure vs Depth (Left Axis)
  Series1.DataSource := SeriesTextSource1;
  Series1.VertAxis := aBothVertAxis;

  with SeriesTextSource1 do
  begin
    FileName := 'C:\Code_Test_Progs\Sync_Axes\AutoScale\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

// Load WHP vs Depth (Left Axis)
  Series2.DataSource := SeriesTextSource2;
  Series2.VertAxis := aBothVertAxis;

  with SeriesTextSource2 do
  begin
    FileName := 'C:\Code_Test_Progs\Sync_Axes\AutoScale\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',4);
    AddField('Y',1);
    Load;
  end;

// Load Temperature vs Depth (Left Axis)
  Series3.DataSource := SeriesTextSource3;
  Series3.VertAxis := aBothVertAxis;

  with SeriesTextSource3 do
  begin
    FileName := 'C:\Code_Test_Progs\Sync_Axes\AutoScale\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',2);
    AddField('Y',1);
    Load;
  end;

  DBChart1.Axes.Bottom.Automatic := False;
  DBChart1.Axes.Bottom.SetMinMax(25, 110);
  DBChart1.Axes.Top.Automatic := False;
  DBChart1.Axes.Top.SetMinMax(0, 320);

  DBChart1.Axes.Right.Labels := False;
  DBChart1.Axes.Top.MinorTickCount := 0;
  DBChart1.Axes.Bottom.MinorTickCount := 0;

end;

// ========== TfrmPTChart.cbxSyncDepClick ======================================
// Doesn't work very well
procedure TfrmPTChart.cbxSyncDepClick(Sender: TObject);
begin
  if cbxSyncDep.Checked then
    SyncTopAxis;
  ApplyChartLogLabels(True);
end;

// ========== TfrmPTChart.cbxAutoBottomClick ===================================
procedure TfrmPTChart.cbxAutoBottomClick(Sender: TObject);
begin
  DBChart1.Axes.Bottom.Automatic := cbxAutoBottom.Checked;
  if not DBChart1.Axes.Bottom.Automatic then
    DBChart1.Axes.Bottom.SetMinMax(25, 110);
end;

// ========== TfrmPTChart.cbxAutoTopClick ======================================
procedure TfrmPTChart.cbxAutoTopClick(Sender: TObject);
begin
  DBChart1.Axes.Top.Automatic := cbxAutoTop.Checked;
  if not DBChart1.Axes.Top.Automatic then
    DBChart1.Axes.Top.SetMinMax(0, 320);
end;

// ========== TfrmPTChart.cbxInvertBottomClick =================================
procedure TfrmPTChart.cbxInvertBottomClick(Sender: TObject);
begin
  DBChart1.Axes.Bottom.Inverted := cbxInvertBottom.Checked;
end;

// ========== TfrmPTChart.cbxInvertTopClick ====================================
procedure TfrmPTChart.cbxInvertTopClick(Sender: TObject);
begin
  DBChart1.Axes.Top.Inverted := cbxInvertTop.Checked;
end;

// ========== TfrmPTChart.cbxLogBottomClick ====================================
procedure TfrmPTChart.cbxLogBottomClick(Sender: TObject);
begin
  DBChart1.Axes.Bottom.Logarithmic := cbxLogBottom.Checked;
  ApplyChartLogLabels(True);
  if DBChart1.Axes.Bottom.Logarithmic then
    DBChart1.Axes.Bottom.Axis.Color := clLime
  else
    DBChart1.Axes.Bottom.Axis.Color := clBlack;
end;

// ========== TfrmPTChart.cbxLogTopClick =======================================
procedure TfrmPTChart.cbxLogTopClick(Sender: TObject);
begin
  DBChart1.Axes.Top.Logarithmic := cbxLogTop.Checked;
  ApplyChartLogLabels(True);
  if DBChart1.Axes.Top.Logarithmic then
    DBChart1.Axes.Top.Axis.Color := clLime
  else
    DBChart1.Axes.Top.Axis.Color := clBlack;
end;

// ========== TfrmPTChart.DBChart1BeforeDrawAxes ===============================
// Now works on zoom, scroll etc
procedure TfrmPTChart.DBChart1BeforeDrawAxes(Sender: TObject);
var
  iInc: integer;

begin
  if cbxSyncDep.Checked then
    SyncTopAxis;
end;

// ========== TfrmPTChart.DBChart1GetAxisLabel =================================
procedure TfrmPTChart.DBChart1GetAxisLabel(Sender: TChartAxis;
  Series: TChartSeries; ValueIndex: Integer; var LabelText: string);
var
  tmp: Double;
  iInc: integer;

begin
end;

// ========== TfrmPTChart.DBChart1Scroll =======================================
procedure TfrmPTChart.DBChart1Scroll(Sender: TObject);
var
  j: integer;

begin
  ApplyChartLogLabels(false);
end;

// ========== TfrmPTChart.DBChart1UndoZoom =====================================
procedure TfrmPTChart.DBChart1UndoZoom(Sender: TObject);
begin
  ApplyChartlogLabels(False)
end;

// ========== TfrmPTChart.SyncTopAxis ==========================================
// Uses bottom axis increments to locate top axis labels
procedure TfrmPTChart.SyncTopAxis;
var
  i, iMin, iMax, Factor: integer;
  d: double;
  Tval, TPos, TMin, TMax, TDiff, BMin, BMax, BDiff, BInc: double;

begin
  Factor := 1;

  DBChart1.Axes.Top.Items.Clear;
  if DBChart1.Axes.Top.Automatic then
  begin
    TMin := DBChart1.Axes.Top.Minimum;
    TMax := DBChart1.Axes.Top.Maximum;
  end
  else
  begin
    TMin := 0;        // arbitrary values
    TMax := 350;
  end;
  TDiff := TMax - Tmin;

  if DBChart1.Axes.Bottom.Automatic then
  begin
    BMin := DBChart1.Axes.Bottom.Minimum;
    BMax := DBChart1.Axes.Bottom.Maximum;
  end
  else
  begin
    BMin := 25;
    BMax := 110;
  end;

  BDiff := BMax - BMin;
  BInc := DBChart1.Axes.Bottom.CalcIncrement;

  if (BInc < 0.01)  then
    Factor := 1000
  else if (BInc < 0.1) then
    Factor := 100
  else if (BInc < 1) then
    Factor := 10;

  iMin := Round(BMin/BInc);
  iMax := Round(BMax/BInc);

  for i := iMin to iMax do
  begin
    Tval := Round(((i * BInc - BMin)/BDiff*TDiff + TMin)*Factor)/Factor;
    TPos := (i * BInc - Bmin)/BDiff*TDiff + TMin;
    DBChart1.Axes.Top.Items.Add(TPos, FloatToStr(Tval));  // adding custom label
  end;
end;

// ========== TfrmPTChart.ApplyChartLogLabels ==================================
procedure TfrmPTChart.ApplyChartLogLabels (bAuto: boolean);
var
  iInc: integer;

begin
  iInc := 7;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Top, iInc, bAuto);

  iInc := 10;
  SetLength(AVal, iInc);
  CalculateLabels(DBChart1.Axes.Bottom, iInc, bAuto);
end;

// ========== TfrmPTChart.CalculateLabels ======================================
procedure TfrmPTChart.CalculateLabels(AAxis: TChartAxis; AInc: integer; bAuto: boolean);
var
  i, j: integer;
  LMin, LMax, LVal, LLog: double;
  LLabel: string;

begin
  if AAxis.Logarithmic then
  begin
    AAxis.Items.Clear;
    if AAxis.Automatic then
      AAxis.CalcMinMax(LMin,LMax)
    else
    begin
    LMin := AAxis.Minimum;
    LMax := AAxis.Maximum;
    end;
    GenerateLabelArray(AInc);
    for i := -6 to 10 do
      for j := 0 to AInc - 1 do
      begin
        LVal := AVal[j] * System.Math.power(10, i);
        if (LVal > LMin) and (LVal < LMax) then
        begin
          LLabel:=FloatToStr(LVal);
          AAxis.Items.Add(LVal, LLabel);
        end;
      end;
  end
  else
  if bAuto then
  begin
    AAxis.Items.Automatic := true;
    AAxis.Labels := true;
  end;
end;


// ========== TfrmPTChart.GenerateLabelArray =================================
procedure TfrmPTChart.GenerateLabelArray(AInc: integer);

begin
  case AInc of
    1:
      begin
        AVal[0] := 1;
      end;
    2:
      begin
        AVal[0] := 1;
        AVal[1] := 3;
      end;
    3:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 5;
      end;
    4:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 3;
        AVal[3] := 5;
      end;
    5:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2.5;
        AVal[3] := 4;
        AVal[4] := 6;
      end;
    6:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 5;
        AVal[5] := 7;
      end;
    7:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 4;
        AVal[5] := 5;
        AVal[6] := 7;
      end;
    8:
      begin
        AVal[0] := 1;
        AVal[1] := 1.5;
        AVal[2] := 2;
        AVal[3] := 3;
        AVal[4] := 4;
        AVal[5] := 5;
        AVal[6] := 6;
        AVal[7] := 8;
      end;
    9:
      begin
        AVal[0] := 1;
        AVal[1] := 2;
        AVal[2] := 3;
        AVal[3] := 4;
        AVal[4] := 5;
        AVal[5] := 6;
        AVal[6] := 7;
        AVal[7] := 8;
        AVal[8] := 9;
      end;
  else
    begin
      AVal[0] := 1;
      AVal[1] := 1.5;
      AVal[2] := 2;
      AVal[3] := 3;
      AVal[4] := 4;
      AVal[5] := 5;
      AVal[6] := 6;
      AVal[7] := 7;
      AVal[8] := 8;
      AVal[9] := 9;
    end;
  end;
end;

end.
